/*
 * Name:    make_flat.c
 *
 * Purpose: Used to generate an area detector image with all of the
 *          pixels set to the same value.
 *
 * Author:  William Lavender
 *
 *------------------------------------------------------------------------
 *
 * Copyright 2007, 2009, 2012, 2014 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "mx_util.h"
#include "mx_stdint.h"
#include "mx_bit.h"
#include "mx_image.h"

int
main( int argc, char *argv[] )
{
	MX_IMAGE_FRAME *flat_frame;
	char *flat_filename;
	uint8_t  *grey8_array;
	uint16_t *grey16_array;
	uint32_t *grey32_array;
	float    *float_array;
	double   *double_array;
	uint8_t  grey8_value;
	uint16_t grey16_value;
	uint32_t grey32_value;
	float    float_value;
	double   double_value;
	unsigned long row_width, column_height, format;
	unsigned long i, image_pixels, image_bytes, bytes_per_pixel;
	double image_value;
	mx_status_type mx_status;

	if ( argc != 6 ) {
		fprintf(stderr,
"\n"
"Usage: make_flat format row_width column_height value flat_filename\n"
"\n"
"  Where format can be 'grey8', 'grey16', 'grey32', 'float', or 'double'.\n"
"\n"
			);
		exit(1);
	}

	flat_frame   = NULL;

	if ( strcmp( argv[1], "grey8" ) == 0 ) {
		format = MXT_IMAGE_FORMAT_GREY8;
		bytes_per_pixel = 1;
	} else
	if ( strcmp( argv[1], "grey16" ) == 0 ) {
		format = MXT_IMAGE_FORMAT_GREY16;
		bytes_per_pixel = 2;
	} else
	if ( strcmp( argv[1], "grey32" ) == 0 ) {
		format = MXT_IMAGE_FORMAT_GREY32;
		bytes_per_pixel = 4;
	} else
	if ( strcmp( argv[1], "float" ) == 0 ) {
		format = MXT_IMAGE_FORMAT_FLOAT;
		bytes_per_pixel = 4;
	} else
	if ( strcmp( argv[1], "double" ) == 0 ) {
		format = MXT_IMAGE_FORMAT_DOUBLE;
		bytes_per_pixel = 8;
	} else {
		fprintf(stderr, "make_flat: Illegal format '%s'\n", argv[1] );
		exit(1);
	}

	row_width = atol( argv[2] );
	column_height = atol( argv[3] );
	image_value = atof( argv[4] );

	flat_filename = argv[5];

	/* Create a new empty frame with the requested dimensions. */

	image_pixels = row_width * column_height;

	image_bytes = image_pixels * bytes_per_pixel;

	mx_status = mx_image_alloc( &flat_frame,
					row_width,
					column_height,
					format,
					mx_native_byteorder(),
					(double) bytes_per_pixel,
					0,
					image_bytes );

	if ( mx_status.code != MXE_SUCCESS )
		exit(1);

	switch( format ) {
	case MXT_IMAGE_FORMAT_GREY8:
		grey8_array = flat_frame->image_data;

		grey8_value = mx_round( image_value );

		for ( i = 0; i < image_pixels; i++ ) {
			grey8_array[i] = grey8_value;
		}
		break;
	case MXT_IMAGE_FORMAT_GREY16:
		grey16_array = flat_frame->image_data;

		grey16_value = mx_round( image_value );

		for ( i = 0; i < image_pixels; i++ ) {
			grey16_array[i] = grey16_value;
		}
		break;
	case MXT_IMAGE_FORMAT_GREY32:
		grey32_array = flat_frame->image_data;

		grey32_value = mx_round( image_value );

		for ( i = 0; i < image_pixels; i++ ) {
			grey32_array[i] = grey32_value;
		}
		break;
	case MXT_IMAGE_FORMAT_FLOAT:
		float_array = flat_frame->image_data;

		float_value = mx_round( image_value );

		for ( i = 0; i < image_pixels; i++ ) {
			float_array[i] = float_value;
		}
		break;
	case MXT_IMAGE_FORMAT_DOUBLE:
		double_array = flat_frame->image_data;

		double_value = mx_round( image_value );

		for ( i = 0; i < image_pixels; i++ ) {
			double_array[i] = double_value;
		}
		break;
	}

	mx_status = mx_image_write_smv_file( flat_frame,
					MXT_IMAGE_FILE_SMV,
					flat_filename );

	if ( mx_status.code != MXE_SUCCESS )
		exit(1);

	fprintf( stderr,
	"Flat frame '%s' successfully created with value = %g.\n",
		flat_filename, image_value );

	exit(0);
}

