/*
 * Name:    smv_bias.c
 *
 * Purpose: Creates an SMV format file where the pixels for each sector
 *          has the same value as for the corresponding sector of a
 *          supplied SMV image file.
 *
 * Author:  William Lavender
 *
 *------------------------------------------------------------------------
 *
 * Copyright 2007, 2012 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */

#define SMV_BIAS_DEBUG	TRUE

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include "mx_util.h"
#include "mx_stdint.h"
#include "mx_array.h"
#include "mx_image.h"

int
main( int argc, char *argv[] )
{
	MX_IMAGE_FRAME *source_frame, *destination_frame;
	char *source_filename, *destination_filename;
	unsigned long source_row_binning, source_column_binning;
	unsigned long sectors_per_row, sectors_per_column;
	unsigned long row_width, column_height;
	unsigned long sector_row_width, sector_column_height;
	unsigned long pixels_per_sector;
	unsigned long srow, scol;
	unsigned long row_start, row_end, col_start, col_end;
	unsigned long row, col, bias_value;
	double sum;
	double source_bytes_per_pixel;
	uint16_t **source_array, **destination_array;
	long dimension[2];
	size_t element_size[2];
	size_t image_bytes;
	mx_status_type mx_status;

	if ( argc != 5 ) {
		fprintf(stderr, "Usage: smv_bias sectors_per_row "
				"sectors_per_column source_file dest_file\n" );
		exit(1);
	}

	sectors_per_row = atol( argv[1] );
	sectors_per_column = atol( argv[2] );
	source_filename = argv[3];
	destination_filename = argv[4];

	/* Load the initial SMV file. */

	source_frame = malloc( sizeof(MX_IMAGE_FRAME) );

	if ( source_frame == NULL ) {
		fprintf( stderr,
		"Could not allocate memory for an MX_IMAGE_FRAME structure.\n");
		exit(1);
	}

	fprintf( stderr, "Reading source file '%s' ... ", source_filename );

	mx_status = mx_image_read_smv_file( &source_frame,
					MXT_IMAGE_FILE_SMV,
					source_filename );

	if ( mx_status.code != MXE_SUCCESS )
		exit(1);

	fprintf( stderr, "Succeeded.\n" );

	/* Get some parameters from the image header. */

	row_width     = MXIF_ROW_FRAMESIZE(source_frame);
	column_height = MXIF_COLUMN_FRAMESIZE(source_frame);

	source_bytes_per_pixel = MXIF_BYTES_PER_PIXEL(source_frame);

	if ( fabs( source_bytes_per_pixel - 2 ) > 0.001 ) {
		fprintf( stderr, "Source frame '%s' has %f bytes per pixel.  "
			"Only 2 bytes per pixel are supported.\n",
			source_filename, source_bytes_per_pixel );
		exit(1);
	}

	/* If the row and column framesizes are not an integer multiple
	 * of the numbers of sectors per row or column, then we cannot
	 * handle this image.
	 */

	if ( row_width % sectors_per_row != 0 ) {
		fprintf( stderr, "The width of '%s' in pixels (%lu) is not an "
		"integer multiple of the number of sectors per row (%lu).\n",
			source_filename, row_width, sectors_per_row );
		exit(1);
	}

	if ( column_height % sectors_per_column != 0 ) {
		fprintf( stderr, "The height of '%s' in pixels (%lu) is not an "
		"integer multiple of the number of sectors per column (%lu).\n",
			source_filename, column_height, sectors_per_column );
		exit(1);
	}

	/* Create a 2-dimensional array overlay for the 1-dimensional
	 * source_frame->image_data pointer.
	 */

	dimension[0] = column_height;
	dimension[1] = row_width;

	element_size[0] = sizeof(uint16_t);
	element_size[1] = sizeof(uint16_t *);

	mx_status = mx_array_add_overlay( source_frame->image_data,
						2, dimension, element_size,
						(void **) &source_array );

	if ( mx_status.code != MXE_SUCCESS )
		exit(1);

	/* Create a new destination array. */

	image_bytes = mx_round( 
	    row_width * column_height * MXIF_BYTES_PER_PIXEL(source_frame) );

	destination_frame = NULL;

	mx_status = mx_image_alloc( &destination_frame,
					row_width,
					column_height,
					MXIF_IMAGE_FORMAT(source_frame),
					MXIF_BYTE_ORDER(source_frame),
					MXIF_BYTES_PER_PIXEL(source_frame),
					0, image_bytes );

	if ( mx_status.code != MXE_SUCCESS )
		exit(1);

	/* Copy the rest of the image header. */

	MXIF_BITS_PER_PIXEL(destination_frame) =
			MXIF_BITS_PER_PIXEL(source_frame);

	MXIF_EXPOSURE_TIME_SEC(destination_frame) =
			MXIF_EXPOSURE_TIME_SEC(source_frame);

	MXIF_EXPOSURE_TIME_NSEC(destination_frame) =
			MXIF_EXPOSURE_TIME_NSEC(source_frame);

	MXIF_TIMESTAMP_SEC(destination_frame) =
			MXIF_TIMESTAMP_SEC(source_frame);

	MXIF_TIMESTAMP_NSEC(destination_frame) =
			MXIF_TIMESTAMP_NSEC(source_frame);

	source_row_binning = MXIF_ROW_BINSIZE(source_frame);
	source_column_binning = MXIF_COLUMN_BINSIZE(source_frame);

	if ( source_row_binning == 0 )
		source_row_binning = 1;

	if ( source_column_binning == 0 )
		source_column_binning = 1;

	MXIF_ROW_BINSIZE(destination_frame) = source_row_binning;
	MXIF_COLUMN_BINSIZE(destination_frame) = source_column_binning;

	/* Create a 2-dimensional array overlay for the 1-dimensional
	 * destination_frame->image_data pointer.
	 */

	dimension[0] = column_height;
	dimension[1] = row_width;

	element_size[0] = sizeof(uint16_t);
	element_size[1] = sizeof(uint16_t *);

	mx_status = mx_array_add_overlay( destination_frame->image_data,
						2, dimension, element_size,
						(void **) &destination_array );

	if ( mx_status.code != MXE_SUCCESS )
		exit(1);

	/* Loop over the sectors, compute the average for each sector,
	 * and then write the average to the corresponding sector of
	 * the destination frame.
	 */

	sector_row_width = row_width / sectors_per_row;
	sector_column_height = column_height / sectors_per_column;

	pixels_per_sector = sector_row_width * sector_column_height;

	fprintf( stderr, "Computing averages ... " );

#if SMV_BIAS_DEBUG
	fprintf( stderr, "\n" );
#endif

	for ( srow = 0; srow < sectors_per_row; srow++ ) {
		for ( scol = 0; scol < sectors_per_column; scol++ ) {
			fprintf( stderr, "srow = %lu, scol = %lu\n",
				srow, scol );

			row_start = srow * sector_column_height;
			col_start = scol * sector_row_width;

			row_end = row_start + sector_column_height;
			col_end = col_start + sector_row_width;

#if SMV_BIAS_DEBUG
			fprintf( stderr, "row_start = %lu, row_end = %lu, "
				"col_start = %lu, col_end = %lu\n",
				row_start, row_end, col_start, col_end );
#endif

			sum = 0.0;

			for ( row = row_start; row < row_end; row++ ) {
				for ( col = col_start; col < col_end; col++ ) {
					sum = sum + source_array[row][col];
				}
			}

			bias_value = mx_round( sum / pixels_per_sector );

#if SMV_BIAS_DEBUG
			fprintf( stderr, "sum = %g, bias_value = %lu\n",
				sum, bias_value );
#endif

			for ( row = row_start; row < row_end; row++ ) {
				for ( col = col_start; col < col_end; col++ ) {
					destination_array[row][col]
						= bias_value;
				}
			}
		}
	}

	fprintf( stderr, "Succeeded.\n" );

	fprintf( stderr, "Writing destination file '%s' ... ",
						destination_filename );

	mx_status = mx_image_write_smv_file( destination_frame,
						MXT_IMAGE_FILE_SMV,
						destination_filename );

	if ( mx_status.code != MXE_SUCCESS )
		exit(1);

	fprintf( stderr, "Succeeded.\n" );

	exit(0);
}

